// ==========================================================================
// Time-stamp: <25 Sep 01 16:15:40 Jean-Guillaume.Dumas@imag.fr>
// ==========================================================================
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>


double e = exp(1);


template<typename precision>
precision rec_croissante(long m) {
    if (m == 0) return e-1.0;
    if (m == 1) return 1.0;
    return e-precision(m)*rec_croissante<precision>(m-1);
}

    

template<typename precision>
precision rec_decroissante(long m, long m0) {
    if (m == m0) return 0;
    return (e-rec_decroissante<precision>(m+1,m0))/(precision(m+1));
}




double trapezes(long m, long n) {
    double res = 0;
    for(double i = 1 ; i < n; ++i)
        res += pow(i/double(n),m)*exp(i/double(n));
    res += e/double(2);
    return res/double(n);
}

    
double simpson(long m, long n) {
    double res = 0, res1 = 0, res2 = 0;
    for(double i = 1 ; i < n; ++i)
        res1 += pow(i/double(n),m)*exp(i/double(n));
    for(double i = 0 ; i < n; ++i)
        res2 += pow((i+0.5)/double(n),m)*exp((i+0.5)/double(n));
    res += e+2.0*res1+4.0*res2;
    return res/double(6*n);
}

double serie(long m, long n) {
    double res = 1.0/(double(m+1.0)), facti = 1.0;
    
    for(double i = 1 ; i < n; ++i) {
        facti *= i;
        res += 1.0/facti/double(m+i+1);
    }
    return res;
}

    
    



int main(int argc, char ** argv) {


    long m = 20;
    
    if (argc > 1) m = atoi(argv[1]);
    

    printf("Calcul de intgrale de x^%ld exp(x) entre 0 et 1\n",m);
    


    printf("\t Rcurrence croissante (simple precision) \t\t : %.20f\n", rec_croissante<float>(m));
    printf("\t Rcurrence croissante (double precision) \t\t : %.20f\n", rec_croissante<double>(m));
    

    cerr << endl;
    
    long m0 = m + 10;
    

    printf("\t Rcurrence dcroissante (simple precision), m0=%ld \t : %.20f\n", m0, rec_decroissante<float>(m,m0));
    printf("\t Rcurrence dcroissante (double precision), m0=%ld \t : %.20f\n", m0, rec_decroissante<double>(m,m0), m0);
    

    m0 += 10;
    

    printf("\t Rcurrence dcroissante (simple precision), m0=%ld \t : %.20f\n", m0, rec_decroissante<float>(m,m0));
    printf("\t Rcurrence dcroissante (double precision), m0=%ld \t : %.20f\n", m0, rec_decroissante<double>(m,m0));
    

    cerr << endl;
    
    long N = 100;
    

    printf("\t Trapzes (double precision), N=%ld \t\t\t : %.20f\n", N, trapezes(m,N));
    
    N*=10;
    printf("\t Trapzes (double precision), N=%ld \t\t\t : %.20f\n", N, trapezes(m,N));

    N*=10;
    printf("\t Trapzes (double precision), N=%ld \t\t\t : %.20f\n", N, trapezes(m,N));
    
    N*=10;
    printf("\t Trapzes (double precision), N=%ld \t\t\t : %.20f\n", N, trapezes(m,N));
    

    cerr << endl;
    

    
    N = 100;
    

    printf("\t Simpson (double precision), N=%ld \t\t\t : %.20f\n", N, simpson(m,N));
    
    N*=10;
    printf("\t Simpson (double precision), N=%ld \t\t\t : %.20f\n", N, simpson(m,N));

    N*=10;
    printf("\t Simpson (double precision), N=%ld \t\t\t : %.20f\n", N, simpson(m,N));
    
    N*=10;
    printf("\t Simpson (double precision), N=%ld \t\t\t : %.20f\n", N, simpson(m,N));
    

    cerr << endl;
    

    
    N = 10;
    

    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));
    
    N += 5;
    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));

    N += 2;
    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));
    
    N += 1;
    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));
    
    N += 1;
    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));

    N += 1;
    printf("\t Srie (double precision), N=%ld \t\t\t : %.20f\n", N, serie(m,N));

    cerr << endl;
    

    

    


    return 0;
}

    
